/*
 * Project name:
     Thumbstick click - Thumbstick Click demo
 * Copyright:
     (c) mikroElektronika, 2014.
 * Revision History:
     20140723:
       - Initial release (DR & TP);
 * Description:
     This is a simple project which demonstrates the use of Thumbstick click board.
     Input device for 360 degree directional movement. TFT shows values.
     The joystick unit receives directional input in the x and y axes by way of two potentiometers connected to a stick. 
     The mechanical pushbutton sends an interrupt signal to the microcontroller. 
     Thumbstick click communicates with the target board through mikroBUS SPI (CS, SCK, MISO and MOSI) lines. 
     The board is designed to use either a 3.3V or a 5V power supply.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
     ext. modules:    Thumbstick click
                      http://www.mikroe.com/click/thumbstick/
                      EasyTFT Board
                      http://www.mikroe.com/eng/products/view/277/various-components/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place Thumbstick Click board in the mikroBUS socket 1.
     - Place EasyTFT and turn on TFT backlight switch SW11.3.
 */

#include "resources.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections


// Thumbstick click module connections
sbit Chip_Select   at GPIOD_ODR.B13;

unsigned int left_right, for_rev, old_left_right, old_for_rev;
unsigned int middle_left_right, middle_for_rev;

char txt[10];

void ExtInt() iv IVT_INT_EXTI15_10 ics ICS_AUTO {
  EXTI_PR.B10 = 1;                     // clear flag
  GPIOD_ODR.B15 = 1;                   // test
}

void InitDisplay(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("Thumbstick  Click", 75, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);

}

// Get ADC values
unsigned int GetADC(unsigned short channel) {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3204
  SPI3_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI3_Read(channel) & 0x0F;               // Get first 8 bits of ADC value
  tmp = tmp << 8;                                // Shift ADC value by 8
  tmp = tmp | SPI3_Read(0);                      // Get remaining 4 bits of ADC value
                                                 // and form 12-bit ADC value
  Chip_Select= 1;                                // Deselect MCP3204
  return tmp;                                    // Returns 12-bit ADC value
}

void DisplayValues() {
  int tmp;
  if(old_left_right != left_right) {
    TFT_Rectangle(70,60,240,90);                 // Clear area
    tmp = middle_left_right - left_right;
    if(tmp < 0) {                                //move right
      WordToStr((unsigned int)(left_right - middle_left_right),&txt);
      TFT_Write_Text("Right:",80,65);
    }else{                                       //move left
      WordToStr((unsigned int)(middle_left_right - left_right),&txt);
      TFT_Write_Text("Left:",80,65);
    }
    TFT_Write_Text(txt,180,65);
    old_left_right = left_right;
  }
  
  if(old_for_rev != for_rev) {
    TFT_Rectangle(70,120,240,150);
    tmp = middle_for_rev - for_rev;
    if(tmp < 0) {                               //move reverse
      WordToStr((unsigned int)(for_rev - middle_for_rev),&txt);
      TFT_Write_Text("Reverse:",80,125);
    }else{                                      //move forward
      WordToStr((unsigned int)(middle_for_rev - for_rev),&txt);
      TFT_Write_Text("Forward:",80,125);
    }
    TFT_Write_Text(txt,180,125);
    old_for_rev = for_rev;
  }

}



void main() {

   // Set Chip Select pin as Output
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13 |  _GPIO_PINMASK_15);
  GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_10); //Thumbstick button on PD10

  // SPI config for Thumbstick click
  SPI3_Init_Advanced(_SPI_FPCLK_DIV128, _SPI_MASTER  | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI3_PC10_11_12);
                     
  InitDisplay();

  //Set middle values
  middle_for_rev = GetADC(0);
  middle_left_right = GetADC(1);

  RCC_APB2ENR.AFIOEN = 1;              // Enable clock for alternate pin functions
  AFIO_EXTICR3 = 0x0300;               // PD10 as External interrupt
  EXTI_RTSR = 0x00000400;              // Set interrupt on Rising edge
  EXTI_IMR |= 0x00000400;              // Set mask
  NVIC_IntEnable(IVT_INT_EXTI15_10);   // Enable External interrupt

  EnableInterrupts();                  // Enables the processor interrupt.

  while(1) {
     for_rev = GetADC(0);
     left_right = GetADC(1);
     DisplayValues();

     if (GPIOD_IDR.B10 == 0) {
         TFT_Rectangle(70,170,240,200);
         TFT_Write_Text("Button   Pressed",100,177);
       }
       else {
         TFT_Rectangle(72,172,238,198);
         TFT_Write_Text("Button   Released",100,177);
       }

     if (GPIOD_ODR.B15 && GPIOD_IDR.B10) {
       GPIOD_ODR.B15 = 0;
     }
     
     delay_ms(10);
  }

}